@@ -14,6 +14,7 @@ from mlflow import tracking
 from mlflow.entities import RunStatus
 from mlflow.exceptions import MlflowException
 from mlflow.projects.submitted_run import SubmittedRun
+from mlflow.projects.utils import _MLFLOW_LOCAL_BACKEND_RUN_ID_CONFIG
 from mlflow.protos.databricks_pb2 import INVALID_PARAMETER_VALUE
 from mlflow.utils import rest_utils, file_utils, databricks_utils
 from mlflow.exceptions import ExecutionException
@@ -161,12 +162,28 @@ class DatabricksJobRunner(object):
         :param command: Shell command to run.
         :param env_vars: Environment variables to set in the process running ``command``.
         :param cluster_spec: Dictionary containing a `Databricks cluster specification
-                             <https://docs.databricks.com/api/latest/jobs.html#clusterspec>`_
-                             to use when launching a run.
+                             <https://docs.databricks.com/dev-tools/api/latest/jobs.html#clusterspec>`_
+                             or a `Databricks new cluster specification
+                             <https://docs.databricks.com/dev-tools/api/latest/jobs.html#jobsclusterspecnewcluster>`_
+                             to use when launching a run. If you specify libraries, this function
+                             will add MLflow to the library list. This function does not support
+                             installation of conda environment libraries on the workers.
         :return: ID of the Databricks job run. Can be used to query the run's status via the
                  Databricks
                  `Runs Get <https://docs.databricks.com/api/latest/jobs.html#runs-get>`_ API.
         """
+        # NB: We use <= on the version specifier to allow running projects on pre-release
+        # versions, where we will select the most up-to-date mlflow version available.
+        # Also note, that we escape this so '<' is not treated as a shell pipe.
+        libraries = [{"pypi": {"package": "'mlflow<=%s'" % VERSION}}]
+
+        # Check syntax of JSON - if it contains libraries and new_cluster, pull those out
+        if 'new_cluster' in cluster_spec:
+            # Libraries are optional, so we don't require that this be specified
+            if 'libraries' in cluster_spec:
+                libraries.extend(cluster_spec['libraries'])
+            cluster_spec = cluster_spec['new_cluster']
+
         # Make jobs API request to launch run.
         req_body_json = {
             'run_name': 'MLflow Run for %s' % project_uri,
@@ -175,10 +192,7 @@ class DatabricksJobRunner(object):
                 'command': command,
                 "env_vars": env_vars
             },
-            # NB: We use <= on the version specifier to allow running projects on pre-release
-            # versions, where we will select the most up-to-date mlflow version available.
-            # Also note, that we escape this so '<' is not treated as a shell pipe.
-            "libraries": [{"pypi": {"package": "'mlflow<=%s'" % VERSION}}],
+            "libraries": libraries,
         }
         run_submit_res = self._jobs_runs_submit(req_body_json)
         databricks_run_id = run_submit_res["run_id"]
@@ -238,6 +252,17 @@ def _get_tracking_uri_for_run():
     return uri
 
 
+def _get_cluster_mlflow_run_cmd(project_dir, run_id, entry_point, parameters):
+    mlflow_run_arr = list(map(shlex_quote, ["mlflow", "run", project_dir,
+                                            "--entry-point", entry_point]))
+    if run_id:
+        mlflow_run_arr.extend(["-c", json.dumps({_MLFLOW_LOCAL_BACKEND_RUN_ID_CONFIG: run_id})])
+    if parameters:
+        for key, value in parameters.items():
+            mlflow_run_arr.extend(["-P", "%s=%s" % (key, value)])
+    return mlflow_run_arr
+
+
 def _get_databricks_run_cmd(dbfs_fuse_tar_uri, run_id, entry_point, parameters):
     """
     Generate MLflow CLI command to run on Databricks cluster in order to launch a run on Databricks.
@@ -247,14 +272,9 @@ def _get_databricks_run_cmd(dbfs_fuse_tar_uri, run_id, entry_point, parameters):
     container_tar_path = posixpath.abspath(posixpath.join(DB_TARFILE_BASE,
                                            posixpath.basename(dbfs_fuse_tar_uri)))
     project_dir = posixpath.join(DB_PROJECTS_BASE, tar_hash)
-    mlflow_run_arr = list(map(shlex_quote, ["mlflow", "run", project_dir,
-                                            "--entry-point", entry_point]))
-    if run_id:
-        mlflow_run_arr.extend(["--run-id", run_id])
-    if parameters:
-        for key, value in parameters.items():
-            mlflow_run_arr.extend(["-P", "%s=%s" % (key, value)])
-    mlflow_run_cmd = " ".join(mlflow_run_arr)
+
+    mlflow_run_arr = _get_cluster_mlflow_run_cmd(project_dir, run_id, entry_point, parameters)
+    mlflow_run_cmd = " ".join([shlex_quote(elem) for elem in mlflow_run_arr])
     shell_command = textwrap.dedent("""
     export PATH=$PATH:$DB_HOME/python/bin &&
     mlflow --version &&
