@@ -9,6 +9,9 @@ from mlflow.exceptions import MlflowException
 from mlflow.protos.databricks_pb2 import INVALID_PARAMETER_VALUE, RESOURCE_ALREADY_EXISTS, \
     INVALID_STATE, RESOURCE_DOES_NOT_EXIST
 import mlflow.store.db.utils
+from mlflow.store.model_registry import SEARCH_REGISTERED_MODEL_MAX_RESULTS_DEFAULT, \
+    SEARCH_REGISTERED_MODEL_MAX_RESULTS_THRESHOLD
+from mlflow.store.db.db_types import SQLITE
 from mlflow.store.db.base_sql_model import Base
 from mlflow.store.entities.paged_list import PagedList
 from mlflow.store.model_registry.abstract_store import AbstractStore
@@ -205,6 +208,88 @@ class SqlAlchemyStore(AbstractStore):
             return [sql_registered_model.to_mlflow_entity()
                     for sql_registered_model in session.query(SqlRegisteredModel).all()]
 
+    def search_registered_models(self,
+                                 filter_string=None,
+                                 max_results=SEARCH_REGISTERED_MODEL_MAX_RESULTS_DEFAULT,
+                                 order_by=None,
+                                 page_token=None):
+        """
+        Search for registered models in backend that satisfy the filter criteria.
+
+        :param filter_string: A filter string expression. Currently supports a single filter
+                              condition either name of model like ``name = 'model_name'``
+        :param max_results: Maximum number of registered models desired.
+        :param order_by: List of column names with ASC|DESC annotation, to be used for ordering
+                         matching search results.
+                         Note:: This field is currently not supported.
+        :param page_token: Token specifying the next page of results. It should be obtained from
+                            a ``search_registered_models`` call.
+        :return: A PagedList of :py:class:`mlflow.entities.model_registry.RegisteredModel` objects
+                that satisfy the search expressions. The pagination token for the next page can be
+                obtained via the ``token`` attribute of the object.
+        """
+        if order_by:
+            raise NotImplementedError("Order by is not implemented for search registered models.")
+        if max_results > SEARCH_REGISTERED_MODEL_MAX_RESULTS_THRESHOLD:
+            raise MlflowException("Invalid value for request parameter max_results."
+                                  "It must be at most {}, but got value {}"
+                                  .format(SEARCH_REGISTERED_MODEL_MAX_RESULTS_THRESHOLD,
+                                          max_results),
+                                  INVALID_PARAMETER_VALUE)
+
+        parsed_filter = SearchUtils.parse_filter_for_model_registry(filter_string)
+        offset = SearchUtils.parse_start_offset_from_page_token(page_token)
+
+        def compute_next_token(current_size):
+            next_token = None
+            if max_results == current_size:
+                final_offset = offset + max_results
+                next_token = SearchUtils.create_page_token(final_offset)
+            return next_token
+
+        if len(parsed_filter) == 0:
+            conditions = []
+        elif len(parsed_filter) == 1:
+            filter_dict = parsed_filter[0]
+            if filter_dict["comparator"] not in \
+                    SearchUtils.VALID_REGISTERED_MODEL_SEARCH_COMPARATORS:
+                raise MlflowException('Search registered models filter expression only '
+                                      'supports the equality(=) comparator, case-sensitive'
+                                      'partial match (LIKE), and case-insensitive partial '
+                                      'match (ILIKE). Input filter string: %s' % filter_string,
+                                      error_code=INVALID_PARAMETER_VALUE)
+            if filter_dict["key"] == "name":
+                if filter_dict["comparator"] == "LIKE":
+                    conditions = [SqlRegisteredModel.name.like(filter_dict["value"])]
+                elif filter_dict["comparator"] == "ILIKE":
+                    conditions = [SqlRegisteredModel.name.ilike(filter_dict["value"])]
+                else:
+                    conditions = [SqlRegisteredModel.name == filter_dict["value"]]
+            else:
+                raise MlflowException('Invalid filter string: %s' % filter_string,
+                                      error_code=INVALID_PARAMETER_VALUE)
+        else:
+            supported_ops = ''.join(['(' + op + ')' for op in
+                                     SearchUtils.VALID_REGISTERED_MODEL_SEARCH_COMPARATORS])
+            sample_query = f'name {supported_ops} "<model_name>"'
+            raise MlflowException(f'Invalid filter string: {filter_string}'
+                                  'Search registered models supports filter expressions like:' +
+                                  sample_query, error_code=INVALID_PARAMETER_VALUE)
+        with self.ManagedSessionMaker() as session:
+            if self.db_type == SQLITE:
+                session.execute("PRAGMA case_sensitive_like = true;")
+            query = session\
+                .query(SqlRegisteredModel)\
+                .filter(*conditions)\
+                .order_by(SqlRegisteredModel.name.asc())\
+                .limit(max_results)
+            if page_token:
+                query = query.offset(offset)
+            sql_registered_models = query.all()
+            registered_models = [rm.to_mlflow_entity() for rm in sql_registered_models]
+            next_page_token = compute_next_token(len(registered_models))
+            return PagedList(registered_models, next_page_token)
+
     def get_registered_model(self, name):
         """
         :param name: Registered model name.
@@ -282,8 +367,8 @@ class SqlAlchemyStore(AbstractStore):
         try:
             version = int(version)
         except ValueError:
-            raise MlflowException("The version for sql alchemy store must be an integer, got '{}'"
-                                  .format(version))
+            raise MlflowException("Model version must be an integer, got '{}'"
+                                  .format(version), error_code=INVALID_PARAMETER_VALUE)
         conditions = [
             SqlModelVersion.name == name,
             SqlModelVersion.version == version,
